-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

-- Node indices for various parts
local ball_43
local ball_42
local cannon_14
local cannon_9

-- Variables for cooldown and fire control
local cooldown
local lastCheckedFireValue -- Allows using an input device axis as shooting control, not just on/off buttons

-- Function to reset the vehicle's state
local function onReset()
  electrics.values['lifter'] = 0
  electrics.values['lifter_input'] = 0
  electrics.values['shoot'] = 0
  cooldown = 0
  lastCheckedFireValue = 0
  
  -- Initialize node indices
  for k, node in pairs(v.data.nodes) do
    if node.name == "ball_43" then ball_43 = k end
    if node.name == "ball_42" then ball_42 = k end
    if node.name == "cannon_14" then cannon_14 = k end
    if node.name == "cannon_9" then cannon_9 = k end
  end
end

-- Function to update graphics each frame
local function updateGFX(dt) -- dt is the delta time in milliseconds
  -- Update lifter value based on input
  electrics.values['lifter'] = math.min(1, math.max(0, electrics.values['lifter'] + electrics.values['lifter_input'] * dt * 0.2))
  
  -- Handle shooting effects
  if electrics.values['shoot'] == 1 then
    if math.random(0, 500) < (500 - cooldown * 1000) then -- Cannon smoke & fire for 500 ms
      obj:addParticleByNodesRelative(cannon_14, cannon_9, 5, 49, 1, 10)
      obj:addParticleByNodesRelative(cannon_14, cannon_9, 5, 29, 1, 10)
      obj:addParticleByNodesRelative(cannon_14, cannon_9, 5, 37, 1, 10)
    end
    cooldown = cooldown + dt
  end
end

-- Function to handle firing mechanism
local function fire(VALUE)
  if VALUE < 0.9 then
    -- Axis/button is not being pressed down far enough, ignore this event
    lastCheckedFireValue = VALUE
    return
  end
  
  if lastCheckedFireValue >= 0.9 then return end -- Ignore consecutive events after passing the 90% threshold
  if VALUE < lastCheckedFireValue then return end -- Ignore event if user is releasing the button/axis

  lastCheckedFireValue = VALUE
  if electrics.values['shoot'] == 1 then
    -- Notify the user *only once* via gui.message
    gui.message("Cannon ball was already fired")
    return
  end

  electrics.values['shoot'] = 1 -- Trigger actual shooting via thruster
  obj:queueGameEngineLua("extensions.hook('onCannonFired'," .. obj:getID() .. ")")
end

-- Function to adjust the lifter input
local function liftBarrel(value)
  electrics.values.lifter_input = value
end

-- Public interface
M.onInit = onReset
M.onReset = onReset
M.updateGFX = updateGFX
M.fire = fire
M.liftBarrel = liftBarrel

return M